@@ -1,7 +1,7 @@
 %% 
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2005-2022. All Rights Reserved.
+%% Copyright Ericsson AB 2005-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -33,8 +33,6 @@
 	 start_subagent/3, stop_subagent/1, 
 	 start_sub_sup/1,  start_sub_sup/2, 
 
-	 start_node/1, stop_node/1,
-
 	 load_master/1, load_master_std/1, unload_master/1, 
 	 loaded_mibs/0, unload_mibs/1,
 
@@ -131,11 +129,16 @@ init_all(Config) when is_list(Config) ->
     %%
 
     ?IPRINT("init_all -> start sub-agent node"),
-    ?line {ok, SaNode}  = start_node(snmp_sa),
+    Args = ["-s", "snmp_test_sys_monitor", "start", "-s", "global", "sync"],
+
+    {ok, SaPeer, SaNode}  = ?CT_PEER(#{name => ?CT_PEER_NAME(snmp_sa), args => Args}),
+    unlink(SaPeer), %% must unlink, otherwise peer will exit before test case
 
     ?IPRINT("init_all -> start manager node"),
-    ?line {ok, MgrNode} = start_node(snmp_mgr),
+    {ok, MgrPeer, MgrNode} = ?CT_PEER(#{name => ?CT_PEER_NAME(snmp_mgr), args => Args}),
+    unlink(MgrPeer), %% must unlink, otherwise peer will exit before test case
 
+    global:sync(),
 
     %% -- 
     %% Create necessary files ( and dirs ) 
@@ -147,37 +150,37 @@ init_all(Config) when is_list(Config) ->
 
     ?IPRINT("init_all -> create agent dir"),
     AgentDir = join(GroupTopDir, "agent/"), 
-    ?line ok = file:make_dir(AgentDir),
+    ok = file:make_dir(AgentDir),
     ?DBG("init_all -> AgentDir ~p", [AgentDir]),
 
     ?IPRINT("init_all -> create agent db dir"),
     AgentDbDir = join(AgentDir, "db/"), 
-    ?line ok   = file:make_dir(AgentDbDir),
+    ok   = file:make_dir(AgentDbDir),
     ?DBG("init_all -> AgentDbDir ~p", [AgentDbDir]),
 
     ?IPRINT("init_all -> create agent log dir"),
     AgentLogDir = join(AgentDir, "log/"), 
-    ?line ok    = file:make_dir(AgentLogDir),
+    ok    = file:make_dir(AgentLogDir),
     ?DBG("init_all -> AgentLogDir ~p", [AgentLogDir]),
 
     ?IPRINT("init_all -> create agent config dir"),
     AgentConfDir = join(AgentDir, "conf/"), 
-    ?line ok     = file:make_dir(AgentConfDir),
+    ok     = file:make_dir(AgentConfDir),
     ?DBG("init_all -> AgentConfDir ~p", [AgentConfDir]),
 
     ?IPRINT("init_all -> create manager dir"),
     MgrDir   = join(GroupTopDir, "mgr/"), 
-    ?line ok = file:make_dir(MgrDir),
+    ok = file:make_dir(MgrDir),
     ?DBG("init_all -> MgrDir ~p", [MgrDir]),
 
     ?IPRINT("init_all -> create sub-agent dir"),
     SaDir    = join(GroupTopDir, "sa/"), 
-    ?line ok = file:make_dir(SaDir),
+    ok = file:make_dir(SaDir),
     ?DBG("init_all -> SaDir ~p", [SaDir]),
 
     ?IPRINT("init_all -> create sub-agent db dir"),
     SaDbDir  = join(SaDir, "db/"), 
-    ?line ok = file:make_dir(SaDbDir),
+    ok = file:make_dir(SaDbDir),
     ?DBG("init_all -> SaDbDir ~p", [SaDbDir]),
 
     %% MibDir = ?config(mib_dir, Config),
@@ -189,35 +192,37 @@ init_all(Config) when is_list(Config) ->
     %% 
 
     ?IPRINT("init_all -> load mnesia application (local)"),
-    ?line ok = application:load(mnesia),
+    ok = application:load(mnesia),
 
     ?IPRINT("init_all -> load application mnesia on node ~p", [SaNode]),
-    ?line ok = rpc:call(SaNode, application, load, [mnesia]),
+    ok = rpc:call(SaNode, application, load, [mnesia]),
     
     ?IPRINT("init_all -> application mnesia (local): set_env dir"),
-    ?line application_controller:set_env(mnesia, dir, 
+    application_controller:set_env(mnesia, dir,
 					 join(AgentDbDir, "Mnesia1")),
 
     ?IPRINT("init_all -> application mnesia: set_env dir on node ~p", [SaNode]),
-    ?line rpc:call(SaNode, application_controller, set_env, 
+    rpc:call(SaNode, application_controller, set_env,
 		   [mnesia, dir,  join(SaDir, "Mnesia2")]),
 
     ?IPRINT("init_all -> create mnesia schema"),
-    ?line ok = mnesia:create_schema([SaNode, node()]),
+    ok = mnesia:create_schema([SaNode, node()]),
     
     ?IPRINT("init_all -> start application mnesia (local)"),
-    ?line ok = application:start(mnesia),
+    ok = application:start(mnesia),
 
     ?IPRINT("init_all -> start application mnesia on ~p", [SaNode]),
-    ?line ok = rpc:call(SaNode, application, start, [mnesia]),
+    ok = rpc:call(SaNode, application, start, [mnesia]),
 
     ?IPRINT("init_all -> get localhost"),
     Ip = ?LOCALHOST(),
 
     ?IPRINT("init_all -> done when"
             "~n   Nodes: ~p", [nodes()]),
-    [{snmp_sa,        SaNode}, 
-     {snmp_mgr,       MgrNode}, 
+    [{snmp_sa,        SaNode},
+     {snmp_sa_peer,   SaPeer},
+     {snmp_mgr,       MgrNode},
+     {snmp_mgr_peer,  MgrPeer},
      {snmp_master,    node()}, 
      {agent_dir,      AgentDir ++ "/"},
      {agent_db_dir,   AgentDbDir ++ "/"},
@@ -237,14 +242,12 @@ finish_all(Config) when is_list(Config) ->
             "~n   Config: ~p"
             "~n   Nodes:  ~p", [Config, nodes()]),
 
-    SaNode  = ?config(snmp_sa, Config),
-    MgrNode = ?config(snmp_mgr, Config),
+    SaPeer  = ?config(snmp_sa_peer, Config),
+    MgrPeer = ?config(snmp_mgr_peer, Config),
 
-    ?IPRINT("finish_all -> stop sub-agent node ~p", [SaNode]),
-    stop_node(SaNode),
+    peer:stop(SaPeer),
 
-    ?IPRINT("finish_all -> stop manager node ~p", [MgrNode]),
-    stop_node(MgrNode),
+    peer:stop(MgrPeer),
 
     ?IPRINT("finish_all -> stop mnesia application"),
     application:stop(mnesia),
@@ -296,6 +299,8 @@ init_case(Config) when is_list(Config) ->
     put(masterip,    tuple_to_list(MasterIP)),
     put(sip,         tuple_to_list(SIP)),
     put(ipfamily,    IpFamily),
+
+    put(receive_response_timeout, receive_response_timeout(Config)),
     
     MibDir = ?config(mib_dir, Config),
     put(mib_dir, MibDir),
@@ -316,6 +321,20 @@ init_case(Config) when is_list(Config) ->
     {SaNode, MgrNode, MibDir}.
 
 
+receive_response_timeout(Config) ->
+    case lists:keysearch(snmp_factor, 1, Config) of
+        {value, {snmp_factor, F}} when (F < 4) ->
+            ?SECS(5);
+        {value, {snmp_factor, F}} when (F < 6) ->
+            ?SECS(10);
+        {value, {snmp_factor, F}} when (F < 8) ->
+            ?SECS(15);
+        {value, {snmp_factor, _}} ->
+            ?SECS(20);
+        _ ->
+            ?SECS(10)
+    end.
+
 %%%--------------------------------------------------
 %%% Used to test the standard mib with our
 %%% configuration.
@@ -579,13 +598,13 @@ tc_run(Mod, Func, Args, Opts) ->
 	    ?EPRINT("Failed starting (test) manager: "
                     "~n   ~p", [Reason]),
 	    (catch snmp_test_mgr:stop()),
-	    ?line ?FAIL({mgr_start_error, Reason});
+	    ?FAIL({mgr_start_error, Reason});
 
 	Err ->
 	    ?EPRINT("Failed starting (test) manager: "
                     "~n   ~p", [Err]),
 	    (catch snmp_test_mgr:stop()),
-	    ?line ?FAIL({mgr_start_failure, Err})
+	    ?FAIL({mgr_start_failure, Err})
     end.
 
 %% We have some crap machines that generate this every now and then
@@ -597,7 +616,7 @@ tc_run_skip_check(_Mod, _Func, _Args,
     ?SKIP([{reason, Reason}]);
 %% We have hosts (mostly *very* slooow VMs) that
 %% can timeout anything. Since we are basically
-%% testing communication, we therefor must check
+%% testing communication, we therefore must check
 %% for system events at every failure. Grrr!
 tc_run_skip_check(Mod, Func, Args, Reason, Cat) ->
     SysEvs = snmp_test_global_sys_monitor:events(),
@@ -608,7 +627,7 @@ tc_run_skip_check(Mod, Func, Args, Reason, Cat) ->
                     "~n   ~p~n", [Cat, Reason]),
             ?FAIL({apply_failed, {Mod, Func, Args}, Reason});
         true ->
-            ?WPRINT("apply (~w) catched "
+            ?WPRINT("apply (~w) caught "
                     "when we got system events: "
                     "~n   Reason:     ~p"
                     "~n   Sys Events: ~p"
@@ -670,11 +689,11 @@ start_agent(Config, Vsns, Opts) ->
             "~n   Vsns:   ~p"
             "~n   Opts:   ~p", [node(), Config, Vsns, Opts]),
     
-    ?line AgentLogDir  = ?config(agent_log_dir,      Config),
-    ?line AgentConfDir = ?config(agent_conf_dir,     Config),
-    ?line AgentDbDir   = ?config(agent_db_dir,       Config),
-    ?line SaNode       = ?config(snmp_sa,            Config),
-    ?line InetBackend  = ?config(socket_create_opts, Config),
+    AgentLogDir  = ?config(agent_log_dir,      Config),
+    AgentConfDir = ?config(agent_conf_dir,     Config),
+    AgentDbDir   = ?config(agent_db_dir,       Config),
+    SaNode       = ?config(snmp_sa,            Config),
+    InetBackend  = ?config(socket_create_opts, Config),
 
     Env = app_agent_env_init(
 	    [{versions,         Vsns}, 
@@ -706,26 +725,26 @@ start_agent(Config, Vsns, Opts) ->
     ?DBG("start_agent -> snmp app supervisor: ~p", [AppSup]),
 
     ?IPRINT("start_agent -> try start master agent",[]),
-    ?line Sup = start_sup(Env), 
-    ?line unlink(Sup),
+    Sup = start_sup(Env),
+    unlink(Sup),
     ?DBG("start_agent -> snmp supervisor: ~p", [Sup]),
 
     ?IPRINT("start_agent -> try (rpc) start sub agent on ~p", [SaNode]),
-    ?line SaDir = ?config(sa_dir, Config),
-    ?line {ok, Sub} = start_sub_sup(SaNode, SaDir),
+    SaDir = ?config(sa_dir, Config),
+    {ok, Sub} = start_sub_sup(SaNode, SaDir),
     ?DBG("start_agent -> done", []),
 
-    ?line [{snmp_app_sup, AppSup}, 
+    [{snmp_app_sup, AppSup},
            {snmp_sup,     {Sup, self()}}, 
            {snmp_sub,     Sub} | Config].
 
 
 app_agent_env_init(Env0, Opts) ->
     ?DBG("app_agent_env_init -> unload snmp",[]),
-    ?line application:unload(snmp),
+    application:unload(snmp),
 
     ?DBG("app_agent_env_init -> load snmp",[]),
-    ?line application:load(snmp),
+    application:load(snmp),
 
     ?DBG("app_agent_env_init -> "
 	 "merge or maybe replace (snmp agent) app env",[]),
@@ -1011,7 +1030,12 @@ await_stopped(Pid, Ref) ->
 %% --- start subagent supervisor ---
 
 start_sub_sup(Node, Dir) ->
-    rpc:call(Node, ?MODULE, start_sub_sup, [Dir]).
+    case rpc:call(Node, ?MODULE, start_sub_sup, [Dir]) of
+        {badrpc, _Reason} = BADRPC ->
+            ?SKIP(BADRPC);
+        Result ->
+            Result
+    end.
     
 start_sub_sup(Dir) ->
     ?DBG("start_sub -> entry",[]),
@@ -1037,6 +1061,8 @@ start_subagent(SaNode, RegTree, Mib) ->
     Func   = start_sub_agent,
     Args   = [MA, RegTree, [Mib1]], 
     case rpc:call(SaNode, Mod, Func, Args) of
+        {badrpc, _Reason} = BADRPC ->
+            ?SKIP(BADRPC);
 	{ok, SA} ->
 	    ?DBG("start_subagent -> SA: ~p", [SA]),
 	    {ok, SA};
@@ -1306,7 +1332,7 @@ do_expect(Err, Idx, ExpVBs, To)
   when is_atom(Err) andalso 
        (is_integer(Idx) orelse is_list(Idx) orelse (Idx == any)) ->
     Check = fun(_, R) -> R end,
-    io_format_expect("'get-response' withing ~w ms with"
+    io_format_expect("'get-response' within ~w ms with"
                      "~n      Error:    ~p"
                      "~n      Index:    ~p"
                      "~n      Varbinds: ~p", [To, Err, Idx, ExpVBs]),
@@ -1639,41 +1665,6 @@ get_next_req(Vars) ->
     Response.
 
 
-%% --- start and stop nodes ---
-
-start_node(Name) ->
-    ?IPRINT("start_node -> entry with"
-            "~n   Name: ~p"
-            "~n when"
-            "~n   hostname of this node: ~p",
-            [Name, list_to_atom(?HOSTNAME(node()))]),
-
-    Pa = filename:dirname(code:which(?MODULE)),
-    ?DBG("start_node -> Pa: ~p", [Pa]),
-
-    A = " -pa " ++ Pa ++ 
-        " -s " ++ atom_to_list(snmp_test_sys_monitor) ++ " start" ++ 
-        " -s global sync",
-    case ?START_NODE(Name, A) of
-	{ok, Node} ->
-	    ?DBG("start_node -> Node: ~p", [Node]),
-            global:sync(),
-	    {ok, Node};
-	{error, Reason}  -> 
-	    ?WPRINT("start_node -> failed starting node ~p:"
-                    "~n      Reason: ~p", [Name, Reason]),
-	    ?line ?SKIP({failed_start_node, Reason});
-	Else  -> 
-	    ?EPRINT("start_node -> failed starting node ~p:"
-                    "~n      ~p", [Name, Else]),
-	    ?line ?FAIL(Else)
-    end.
-
-
-stop_node(Node) ->
-    ?IPRINT("stop_node -> Node: ~p", [Node]),
-    ?STOP_NODE(Node).
-
 
 %%%-----------------------------------------------------------------
 %%% Configuration
@@ -1691,13 +1682,13 @@ config(Vsns, MgrDir, AgentConfDir, MIp, AIp, IpFamily) ->
             "~n   AIp:          ~p"
             "~n   IpFamily:     ~p",
             [Vsns, MgrDir, AgentConfDir, MIp, AIp, IpFamily]),
-    ?line {Domain, ManagerAddr} =
+    {Domain, ManagerAddr} =
 	case IpFamily of
 	    inet6 ->
 		TransportDomain6 = transportDomainUdpIpv6,
 		AgentAddr6       = {AIp, 4000},
 		ManagerAddr6     = {MIp, ?TRAP_UDP},
-		?line ok =
+		ok =
 		    snmp_config:write_agent_snmp_files(
 		      AgentConfDir, Vsns,
 		      TransportDomain6, ManagerAddr6, AgentAddr6, "test"),
@@ -1710,32 +1701,32 @@ config(Vsns, MgrDir, AgentConfDir, MIp, AIp, IpFamily) ->
                 %%     [#{addr => {AIp2, 4000}, kind => req_responder},
                 %%      #{addr => {AIp2, 4001}, kind => trap_sender}],
                 AgentPreTransport  = [#{addr => {AIp2, 4000}}],
-		?line ok =
+		ok =
 		    snmp_config:write_agent_snmp_files(
 		      AgentConfDir, Vsns,
 		      TransportDomain4, ManagerAddr4, AgentPreTransport,
                       "test"),
 		{TransportDomain4, ManagerAddr4};
 	    _ ->
-		?line ok =
+		ok =
 		    snmp_config:write_agent_snmp_files(
 		      AgentConfDir, Vsns, MIp, ?TRAP_UDP, AIp, 4000, "test"),
 		{snmpUDPDomain, {MIp, ?TRAP_UDP}}
 	  end,
 
-    ?line case update_usm(Vsns, AgentConfDir) of
+    case update_usm(Vsns, AgentConfDir) of
 	      true ->
-		  ?line copy_file(join(AgentConfDir, "usm.conf"),
+		  copy_file(join(AgentConfDir, "usm.conf"),
 				  join(MgrDir, "usm.conf")),
-		  ?line update_usm_mgr(Vsns, MgrDir);
+		  update_usm_mgr(Vsns, MgrDir);
 	      false ->
-		  ?line ok
+		  ok
 	  end,
-    ?line update_community(Vsns, AgentConfDir),
-    ?line update_vacm(Vsns, AgentConfDir),
-    ?line write_target_addr_conf(AgentConfDir, Domain, ManagerAddr, Vsns),
-    ?line write_target_params_conf(AgentConfDir, Vsns),
-    ?line write_notify_conf(AgentConfDir),
+    update_community(Vsns, AgentConfDir),
+    update_vacm(Vsns, AgentConfDir),
+    write_target_addr_conf(AgentConfDir, Domain, ManagerAddr, Vsns),
+    write_target_params_conf(AgentConfDir, Vsns),
+    write_notify_conf(AgentConfDir),
     ok.
 
 maybe_fix_addr(Addr) when is_list(Addr) ->
@@ -1843,7 +1834,7 @@ update_usm(Vsns, Dir) ->
 		     usmHMACSHAAuthProtocol, "", "", 
 		     usmDESPrivProtocol, "", "", "", 
 		     "passwd_shaxxxxxxxxxx", "passwd_desxxxxxx"}],
-	    ?line ok = snmp_config:update_agent_usm_config(Dir, Conf),
+	    ok = snmp_config:update_agent_usm_config(Dir, Conf),
 	    true;
 	false ->
 	    false
@@ -1862,14 +1853,14 @@ update_usm_mgr(Vsns, Dir) ->
 		     usmDESPrivProtocol, "", "", "", 
 		     "passwd_shaxxxxxxxxxx", "passwd_desxxxxxx"}], 
 
-	    ?line ok = snmp_config:update_agent_usm_config(Dir, Conf),
+	    ok = snmp_config:update_agent_usm_config(Dir, Conf),
 	    true;
 	false ->
 	    false
     end.
 
 rewrite_usm_mgr(Dir, ShaKey, DesKey) -> 
-    ?line ok = file:rename(join(Dir,"usm.conf"),
+    ok = file:rename(join(Dir,"usm.conf"),
 			   join(Dir,"usm.old")),
     Conf = [{"agentEngine", "newUser", "newUser", zeroDotZero, 
 	     usmHMACSHAAuthProtocol, "", "", 
@@ -1877,10 +1868,10 @@ rewrite_usm_mgr(Dir, ShaKey, DesKey) ->
 	    {"mgrEngine", "newUser", "newUser", zeroDotZero, 
 	     usmHMACSHAAuthProtocol, "", "", 
 	     usmDESPrivProtocol, "", "", "", ShaKey, DesKey}], 
-    ?line ok = snmp_config:write_agent_usm_config(Dir, "", Conf).
+    ok = snmp_config:write_agent_usm_config(Dir, "", Conf).
 
 reset_usm_mgr(Dir) ->
-    ?line ok = file:rename(join(Dir,"usm.old"),
+    ok = file:rename(join(Dir,"usm.old"),
 			   join(Dir,"usm.conf")).
 
 
@@ -1888,7 +1879,7 @@ update_community([v3], _Dir) ->
     ok;
 update_community(_, Dir) ->
     Conf = [{"no-rights", "no-rights", "no-rights", "", ""}],
-    ?line ok = snmp_config:update_agent_community_config(Dir, Conf).
+    ok = snmp_config:update_agent_community_config(Dir, Conf).
     
     
 -define(tDescr_instance, [1,3,6,1,2,1,16,1,0]).
@@ -1903,17 +1894,17 @@ update_vacm(_Vsn, Dir) ->
 	    {vacmSecurityToGroup, usm, "newUser",    "initial"},
 	    {vacmViewTreeFamily, "internet", ?tDescr_instance, 
 	     excluded, null}],
-    ?line ok = snmp_config:update_agent_vacm_config(Dir, Conf).
+    ok = snmp_config:update_agent_vacm_config(Dir, Conf).
     
     
 write_community_conf(Dir, Conf) ->
-    ?line ok = snmp_config:write_agent_community_config(Dir, "", Conf).
+    ok = snmp_config:write_agent_community_config(Dir, "", Conf).
 
 write_target_addr_conf(Dir, Conf) ->
-    ?line ok = snmp_config:write_agent_target_addr_config(Dir, "", Conf).
+    ok = snmp_config:write_agent_target_addr_config(Dir, "", Conf).
 
 write_target_addr_conf(Dir, Ip_or_Domain, Port_or_Addr, Vsns) ->
-    ?line ok =
+    ok =
 	snmp_config:write_agent_snmp_target_addr_conf(
 	  Dir, Ip_or_Domain, Port_or_Addr, Vsns).
 
@@ -1930,7 +1921,7 @@ rewrite_target_addr_conf(Dir, NewPort) ->
 	    ok  
     end,
 
-    ?line [TrapAddr|Addrs] = 
+    [TrapAddr|Addrs] =
 	snmp_conf:read(TAFile, fun rewrite_target_addr_conf_check/1),
 
     ?DBG("rewrite_target_addr_conf -> TrapAddr: ~p",[TrapAddr]),
@@ -1939,10 +1930,10 @@ rewrite_target_addr_conf(Dir, NewPort) ->
     
     ?DBG("rewrite_target_addr_conf -> NewAddrs: ~p",[NewAddrs]),
 
-    ?line ok = file:rename(join(Dir,"target_addr.conf"),
+    ok = file:rename(join(Dir,"target_addr.conf"),
 			   join(Dir,"target_addr.old")),
 
-    ?line ok = snmp_config:write_agent_target_addr_config(Dir, "", NewAddrs).
+    ok = snmp_config:write_agent_target_addr_config(Dir, "", NewAddrs).
 
 rewrite_target_addr_conf_check(O) -> 
     {ok,O}.
@@ -1958,7 +1949,7 @@ rewrite_target_addr_conf2(_NewPort,O) ->
     O.
 
 reset_target_addr_conf(Dir) ->
-    ?line ok = file:rename(join(Dir, "target_addr.old"),
+    ok = file:rename(join(Dir, "target_addr.old"),
 			   join(Dir, "target_addr.conf")).
 
 write_target_params_conf(Dir, Vsns) -> 
@@ -1967,28 +1958,28 @@ write_target_params_conf(Dir, Vsns) ->
 	   (v3) -> {"target_v3", v3,  usm, "all-rights", noAuthNoPriv}
 	end,
     Conf = [F(Vsn) || Vsn <- Vsns],
-    ?line ok = snmp_config:write_agent_target_params_config(Dir, "", Conf).
+    ok = snmp_config:write_agent_target_params_config(Dir, "", Conf).
 
 rewrite_target_params_conf(Dir, SecName, SecLevel) 
   when is_list(SecName) andalso is_atom(SecLevel) -> 
-    ?line ok = file:rename(join(Dir,"target_params.conf"),
+    ok = file:rename(join(Dir,"target_params.conf"),
 			   join(Dir,"target_params.old")),
     Conf = [{"target_v3", v3, usm, SecName, SecLevel}],
-    ?line ok = snmp_config:write_agent_target_params_config(Dir, "", Conf).
+    ok = snmp_config:write_agent_target_params_config(Dir, "", Conf).
 
 reset_target_params_conf(Dir) ->
-    ?line ok = file:rename(join(Dir,"target_params.old"),
+    ok = file:rename(join(Dir,"target_params.old"),
 			   join(Dir,"target_params.conf")).
 
 write_notify_conf(Dir) -> 
     Conf = [{"standard trap",   "std_trap",   trap}, 
 	    {"standard inform", "std_inform", inform}],
-    ?line ok = snmp_config:write_agent_notify_config(Dir, "", Conf).
+    ok = snmp_config:write_agent_notify_config(Dir, "", Conf).
 
 write_view_conf(Dir) -> 
     Conf = [{2, [1,3,6], included, null},
 	    {2, ?tDescr_instance, excluded, null}], 
-    ?line ok = snmp_config:write_agent_view_config(Dir, "", Conf).
+    ok = snmp_config:write_agent_view_config(Dir, "", Conf).
 
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
